BACTERIA = "bacteria"
CRYPTOCOCCUS_NEOFORMANS = "cryptococcus_neoformans"
ECOLI = "e_coli"
HEADACHE = "headache"
KLEBSIELLA_SPP = "klebsiella_sp"
LAM_NEG0 = "0"
LAM_POS1 = "1"
LAM_POS2 = "2"
LAM_POS3 = "3"
LAM_POS4 = "4"
LAM_POS5 = "5"
NO_GROWTH = "no_growth"
TB_ACTION = "reportable-tb-result"



# Generated by Django 3.2.11 on 2022-02-24 16:26

import edc_model.validators.date
import edc_protocol.validators
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_microbiology", "0007_auto_20220224_1910"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_afb_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Sputum AFB date",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_afb_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Sputum AFB microscopy performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_afb_result",
            field=models.CharField(
                choices=[("POS", "Positive"), ("NEG", "Negative"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                verbose_name="Sputum AFB results",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_culture_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Sputum culture performed?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_culture_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate / contaminated"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=10,
                verbose_name="Sputum culture results",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_genexpert_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Sputum Gene-Xpert date",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_genexpert_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Sputum Gene-Xpert performed?",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_afb_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Sputum AFB date",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_afb_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=5,
                verbose_name="Sputum AFB microscopy performed?",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_afb_result",
            field=models.CharField(
                choices=[("POS", "Positive"), ("NEG", "Negative"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=10,
                verbose_name="Sputum AFB results",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_culture_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Sputum culture performed?",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_culture_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate / contaminated"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=10,
                verbose_name="Sputum culture results",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_genexpert_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[
                    edc_protocol.validators.date_not_before_study_start,
                    edc_model.validators.date.date_not_future,
                ],
                verbose_name="Sputum Gene-Xpert date",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_genexpert_performed",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                max_length=15,
                verbose_name="Sputum Gene-Xpert performed?",
            ),
        ),
    ]



def get_blood_culture_fieldset():
    return [
        "Blood Culture",
        {
            "fields": (
                "blood_culture_performed",
                "blood_culture_date",
                "blood_culture_day",
                "blood_culture_result",
                "blood_culture_organism",
                "blood_culture_organism_other",
                "blood_culture_bacteria",
                "blood_culture_bacteria_other",
            )
        },
    ]


def get_csf_fieldset():
    return [
        "CSF Microbiology",
        {
            "fields": (
                "csf_genexpert_performed",
                "csf_genexpert_date",
                "csf_genexpert_result",
            )
        },
    ]


def get_histopathology_fieldset():
    return [
        "Histopathology",
        {
            "fields": (
                "tissue_biopsy_performed",
                "tissue_biopsy_date",
                "tissue_biopsy_day",
                "tissue_biopsy_result",
                "tissue_biopsy_organism",
                "tissue_biopsy_organism_other",
                "tissue_biopsy_report",
            )
        },
    ]


def get_sputum_afb_fieldset():
    return [
        "Sputum AFB",
        {
            "fields": (
                "sputum_afb_performed",
                "sputum_afb_date",
                "sputum_afb_result",
            )
        },
    ]


def get_sputum_culture_fieldset():
    return [
        "Sputum culture",
        {
            "fields": (
                "sputum_culture_performed",
                "sputum_culture_date",
                "sputum_culture_result",
            )
        },
    ]


def get_sputum_genexpert_fieldset():
    return [
        "Sputum Gene-Xpert",
        {
            "fields": (
                "sputum_genexpert_performed",
                "sputum_genexpert_date",
                "sputum_genexpert_result",
            )
        },
    ]


def get_urinary_lam_fieldset():
    return [
        "Urinary LAM",
        {
            "fields": (
                "urinary_lam_performed",
                "urinary_lam_date",
                "urinary_lam_result",
                "urinary_lam_result_grade",
            ),
        },
    ]


def get_urine_culture_fieldset():
    return [
        "Urine Culture (Only for patients with >50 white cells in urine)",
        {
            "fields": (
                "urine_culture_performed",
                "urine_culture_date",
                "urine_culture_result",
                "urine_culture_organism",
                "urine_culture_organism_other",
            )
        },
    ]



# Generated by Django 3.2.11 on 2022-02-24 15:26

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("edc_microbiology", "0005_auto_20220224_1509"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalmicrobiology",
            old_name="sputum_culture_taken_date",
            new_name="sputum_culture_date",
        ),
        migrations.RenameField(
            model_name="historicalmicrobiology",
            old_name="urine_culture_taken_date",
            new_name="urine_culture_date",
        ),
        migrations.RenameField(
            model_name="microbiology",
            old_name="sputum_culture_taken_date",
            new_name="sputum_culture_date",
        ),
        migrations.RenameField(
            model_name="microbiology",
            old_name="urine_culture_taken_date",
            new_name="urine_culture_date",
        ),
        migrations.RemoveField(
            model_name="historicalmicrobiology",
            name="sputum_culture_pos_description",
        ),
        migrations.RemoveField(
            model_name="microbiology",
            name="sputum_culture_pos_description",
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_culture_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate / contaminated"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=10,
                verbose_name="Culture results",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="sputum_genexpert_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=45,
                verbose_name="Sputum Gene-Xpert results",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmicrobiology",
            name="urinary_lam_result_grade",
            field=models.CharField(
                choices=[
                    ("1", "1+ (low)"),
                    ("2", "2+"),
                    ("3", "3+"),
                    ("4", "4+"),
                    ("5", "5+ (high)"),
                    ("unknown", "Unknown / Grade not reported"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If Urinary LAM is positive, grade",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_culture_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate / contaminated"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=10,
                verbose_name="Culture results",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="sputum_genexpert_result",
            field=models.CharField(
                choices=[
                    ("POS", "MTB Positive"),
                    ("NEG", "MTB Negative"),
                    ("IND", "Indeterminate"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=45,
                verbose_name="Sputum Gene-Xpert results",
            ),
        ),
        migrations.AlterField(
            model_name="microbiology",
            name="urinary_lam_result_grade",
            field=models.CharField(
                choices=[
                    ("1", "1+ (low)"),
                    ("2", "2+"),
                    ("3", "3+"),
                    ("4", "4+"),
                    ("5", "5+ (high)"),
                    ("unknown", "Unknown / Grade not reported"),
                    ("N/A", "Not applicable"),
                ],
                default="N/A",
                max_length=25,
                verbose_name="If Urinary LAM is positive, grade",
            ),
        ),
    ]



# Generated by Django 3.2.11 on 2022-02-23 19:58

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("edc_microbiology", "0003_auto_20220223_2256"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalmicrobiology",
            old_name="tissue_biopsy_day_taken",
            new_name="tissue_biopsy_day",
        ),
        migrations.RenameField(
            model_name="microbiology",
            old_name="tissue_biopsy_day_taken",
            new_name="tissue_biopsy_day",
        ),
    ]



from edc_consent.form_validators import SubjectConsentFormValidatorMixin
from edc_constants.constants import OTHER, POS, YES
from edc_crf.crf_form_validator import CrfFormValidator
from edc_crf.crf_form_validator_mixins import BaseFormValidatorMixin
from edc_form_validators import FormValidator
from edc_form_validators.extra_mixins import StudyDayFormValidatorMixin

from .constants import BACTERIA


class BloodCultureFormValidatorMixin:
    def validate_blood_culture(self: CrfFormValidator):
        self.required_if(
            YES, field="blood_culture_performed", field_required="blood_culture_date"
        )

        self.applicable_if(
            YES,
            field="blood_culture_performed",
            field_applicable="blood_culture_result",
        )

        self.required_if(POS, field="blood_culture_result", field_required="blood_culture_day")

        self.applicable_if(
            POS,
            field="blood_culture_result",
            field_applicable="blood_culture_organism",
        )

        self.validate_other_specify(
            field="blood_culture_organism",
            other_specify_field="blood_culture_organism_other",
            other_stored_value=OTHER,
        )

        condition = (
            self.cleaned_data.get("blood_culture_organism") == BACTERIA
            or self.cleaned_data.get("blood_culture_organism") == "bacteria_and_cryptococcus"
        )
        self.applicable_if_true(condition=condition, field_applicable="blood_culture_bacteria")

        self.validate_other_specify(
            field="blood_culture_bacteria",
            other_specify_field="blood_culture_bacteria_other",
            other_stored_value=OTHER,
        )


class BloodCultureSimpleFormValidatorMixin:
    def validate_blood_culture(self: CrfFormValidator):
        self.required_if(
            YES, field="blood_culture_performed", field_required="blood_culture_date"
        )

        self.applicable_if(
            YES,
            field="blood_culture_performed",
            field_applicable="blood_culture_result",
        )

        self.required_if(
            POS,
            field="blood_culture_result",
            field_required="blood_culture_organism_text",
        )


class CsfGenexpertFormValidator:
    def validate_csf_genexpert(self: CrfFormValidator):
        self.required_if(
            YES,
            field="csf_genexpert_performed",
            field_required="csf_genexpert_date",
        )

        self.applicable_if(
            YES,
            field="csf_genexpert_performed",
            field_applicable="csf_result_genexpert",
        )


class HistopathologyFormValidatorMixin:
    def validate_histopathology(self: CrfFormValidator, exclude_fields: list[str] = None):
        exclude_fields = exclude_fields or []

        self.required_if(
            YES, field="tissue_biopsy_performed", field_required="tissue_biopsy_date"
        )

        self.applicable_if(
            YES,
            field="tissue_biopsy_performed",
            field_applicable="tissue_biopsy_result",
        )

        if "tissue_biopsy_day" not in exclude_fields:
            self.required_if(
                POS, field="tissue_biopsy_result", field_required="tissue_biopsy_day"
            )

        self.applicable_if(
            POS,
            field="tissue_biopsy_result",
            field_applicable="tissue_biopsy_organism",
        )

        self.validate_other_specify(
            field="tissue_biopsy_organism",
            other_specify_field="tissue_biopsy_organism_other",
            other_stored_value=OTHER,
        )


class UrineCultureFormValidatorMixin:
    def validate_urine_culture(self: CrfFormValidator):
        self.required_if(
            YES, field="urine_culture_performed", field_required="urine_culture_date"
        )

        self.applicable_if(
            YES,
            field="urine_culture_performed",
            field_applicable="urine_culture_result",
        )

        self.applicable_if(
            POS,
            field="urine_culture_result",
            field_applicable="urine_culture_organism",
        )

        self.validate_other_specify(
            field="urine_culture_organism",
            other_specify_field="urine_culture_organism_other",
            other_stored_value=OTHER,
        )


class SputumCultureFormValidatorMixin:
    def validate_sputum_culture(self: CrfFormValidator):
        self.required_if(
            YES, field="sputum_culture_performed", field_required="sputum_culture_date"
        )
        self.applicable_if(
            YES, field="sputum_culture_performed", field_applicable="sputum_culture_result"
        )


class SputumAfbFormValidatorMixin:
    def validate_sputum_afb(self: CrfFormValidator):
        self.required_if(YES, field="sputum_afb_performed", field_required="sputum_afb_date")
        self.applicable_if(
            YES, field="sputum_afb_performed", field_applicable="sputum_afb_result"
        )


class SputumGenexpertFormValidatorMixin:
    def validate_sputum_genexpert(self: CrfFormValidator):
        self.required_if(
            YES,
            field="sputum_genexpert_performed",
            field_required="sputum_genexpert_date",
        )
        self.applicable_if(
            YES,
            field="sputum_genexpert_performed",
            field_applicable="sputum_genexpert_result",
        )


class UrinaryLamFormValidatorMixin:
    def validate_urinary_lam(self: CrfFormValidator):
        self.required_if(
            YES,
            field="urinary_lam_performed",
            field_required="urinary_lam_date",
        )

        self.applicable_if(
            YES,
            field="urinary_lam_performed",
            field_applicable="urinary_lam_result",
        )

        self.applicable_if(
            POS,
            field="urinary_lam_result",
            field_applicable="urinary_lam_result_grade",
        )


class MicrobiologyFormValidatorMixin(
    StudyDayFormValidatorMixin,
    UrinaryLamFormValidatorMixin,
    SputumGenexpertFormValidatorMixin,
    SputumCultureFormValidatorMixin,
    SputumAfbFormValidatorMixin,
    BloodCultureFormValidatorMixin,
    HistopathologyFormValidatorMixin,
    UrineCultureFormValidatorMixin,
):
    def clean(self):
        self.validate_study_day_with_datetime(
            subject_identifier=self.subject_identifier,
            study_day=self.cleaned_data.get("day_blood_taken"),
            compare_date=self.cleaned_data.get("blood_taken_date"),
            study_day_field="day_blood_taken",
        )

        self.validate_study_day_with_datetime(
            subject_identifier=self.subject_identifier,
            study_day=self.cleaned_data.get("day_biopsy_taken"),
            compare_date=self.cleaned_data.get("biopsy_date"),
            study_day_field="day_biopsy_taken",
        )

        self.validate_urinary_lam()

        self.validate_sputum_afb()

        self.validate_sputum_culture()

        self.validate_sputum_genexpert()

        self.validate_blood_culture()

        self.validate_histopathology()

        self.validate_urine_culture()


class MicrobiologyFormValidator(MicrobiologyFormValidatorMixin, CrfFormValidator):
    """Assumes this is a CRF"""

    pass


class MicrobiologyPrnFormValidator(
    MicrobiologyFormValidatorMixin,
    SubjectConsentFormValidatorMixin,
    BaseFormValidatorMixin,
    FormValidator,
):
    """Assumes this is a PRN"""

    pass



from decimal import Decimal

from django import forms
from django.core.exceptions import ValidationError
from django.test import TestCase
from edc_constants.constants import NO, NOT_APPLICABLE, OTHER, POS, YES
from edc_crf.crf_form_validator_mixins import BaseFormValidatorMixin
from edc_form_validators import FormValidator
from edc_registration.models import RegisteredSubject
from edc_utils import get_utcnow
from edc_visit_schedule.constants import DAY1

from edc_microbiology.constants import (
    BACTERIA,
    CRYPTOCOCCUS_NEOFORMANS,
    KLEBSIELLA_SPP,
    NO_GROWTH,
)
from edc_microbiology.form_validators import MicrobiologyFormValidatorMixin

from ..models import Appointment, SubjectVisit


class MicrobiologyFormValidator(
    MicrobiologyFormValidatorMixin,
    BaseFormValidatorMixin,
    FormValidator,
):
    """Assumes this is a PRN"""

    pass


class TestMicrobiologyFormValidator(TestCase):
    def setUp(self):
        self.subject_identifier = "1234"
        RegisteredSubject.objects.create(
            subject_identifier=self.subject_identifier,
            randomization_datetime=get_utcnow(),
        )
        appointment = Appointment.objects.create(
            subject_identifier=self.subject_identifier,
            appt_datetime=get_utcnow(),
            visit_code=DAY1,
            visit_schedule_name="visit_schedule",
            schedule_name="schedule",
            timepoint=Decimal("0.0"),
        )
        self.subject_visit = SubjectVisit.objects.create(
            appointment=appointment, subject_identifier=self.subject_identifier
        )

    def test_urine_culture_performed_yes_require_urine_culture_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_performed": YES,
            "urine_culture_date": get_utcnow(),
            "urine_culture_result": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("urine_culture_result", form_validator._errors)

    def test_urine_culture_performed_no_require_urine_culture_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_performed": NO,
            "urine_culture_result": NO_GROWTH,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_urine_culture_performed_na_given_urine_culture_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_performed": YES,
            "urine_culture_result": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_urine_culture_result_none_urine_culture_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_result": POS,
            "urine_culture_organism": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("urine_culture_organism", form_validator._errors)

    def test_pos_urine_results_na_urine_culture_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_result": POS,
            "urine_culture_organism": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_urine_results_with_urine_culture_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_result": POS,
            "urine_culture_organism": KLEBSIELLA_SPP,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got{e}")

    def test_other_urine_culture_result_require_urine_organism_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "urine_culture_organism": OTHER,
            "urine_culture_organism_other": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("urine_culture_organism_other", form_validator._errors)

    def test_yes_blood_culture_performed_none_blood_culture_results(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_performed": YES,
            "blood_culture_date": get_utcnow(),
            "blood_culture_result": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("blood_culture_result", form_validator._errors)

    def test_no_blood_culture_performed_none_blood_culture_results(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_performed": NO,
            "blood_culture_result": POS,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("blood_culture_result", form_validator._errors)

    def test_no_blood_culture_performed_with_blood_culture_results(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_performed": NO,
            "blood_culture_result": NO_GROWTH,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_blood_culture_results_require_date_blood_taken(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_result": POS,
            "blood_culture_date": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_blood_culture_results_require_blood_culture_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_result": POS,
            "blood_culture_date": get_utcnow().date(),
            "blood_culture_day": 1,
            "blood_culture_organism": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("blood_culture_organism", form_validator._errors)

    def test_pos_blood_culture_results_na_blood_culture_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_result": POS,
            "blood_culture_organism": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_blood_culture_organism_require_culture_organism_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_organism": OTHER,
            "blood_culture_organism_other": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_blood_culture_organism_na_culture_organism_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_organism": CRYPTOCOCCUS_NEOFORMANS,
            "blood_culture_organism_other": CRYPTOCOCCUS_NEOFORMANS,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_blood_culture_organism_with_culture_organism_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_organism": OTHER,
            "blood_culture_organism_other": "other organism",
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got{e}")

    def test_blood_organism_is_bacteria_require_bacteria_identified(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_organism": BACTERIA,
            "blood_culture_bacteria": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("blood_culture_bacteria", form_validator._errors)

    def test_blood_organism_is_bacteria_na_bacteria_identified(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_organism": NOT_APPLICABLE,
            "blood_culture_bacteria": KLEBSIELLA_SPP,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("blood_culture_bacteria", form_validator._errors)

    def test_other_bacteria_identified_require_bacteria_identified_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_bacteria": OTHER,
            "blood_culture_bacteria_other": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_bacteria_identified_na_bacteria_identified_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "blood_culture_bacteria": OTHER,
            "blood_culture_bacteria_other": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_sputum_results_culture_require_sputum_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "sputum_culture_performed": YES,
            "sputum_culture_result": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_sputum_results_culture_with_sputum_results_positive(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "sputum_culture_result": POS,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got{e}")

    def test_tissue_biopsy_performed_require_tissue_biopsy_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": YES,
            "tissue_biopsy_result": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_no_tissue_biopsy_performed_none_tissue_biopsy_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": NO,
            "tissue_biopsy_result": POS,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("tissue_biopsy_result", form_validator._errors)

    def no_test_tissue_biopsy_performed_with_tissue_biopsy_result(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": NO,
            "tissue_biopsy_result": NO_GROWTH,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_tissue_biopsy_result_none_biopsy_date(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": NO,
            "tissue_biopsy_date": get_utcnow().date(),
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_tissue_biopsy_result_na_tissue_biopsy_date(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": YES,
            "tissue_biopsy_date": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)
        self.assertIn("tissue_biopsy_date", form_validator._errors)

    def test_pos_tissue_biopsy_result_with_tissue_biopsy_day(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": YES,
            "tissue_biopsy_date": get_utcnow(),
            "tissue_biopsy_result": POS,
            "tissue_biopsy_day": None,
        }
        form = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form.validate)
        self.assertIn("tissue_biopsy_day", form._errors)

    def test_pos_tissue_biopsy_result_none_tissue_biopsy_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_performed": YES,
            "tissue_biopsy_date": get_utcnow(),
            "tissue_biopsy_result": POS,
            "tissue_biopsy_organism": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_pos_tissue_biopsy_result_na_tissue_biopsy_organism(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_result": POS,
            "tissue_biopsy_organism": NOT_APPLICABLE,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_tissue_biopsy_org_none_tissue_biopsy_org_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_organism": OTHER,
            "tissue_biopsy_organism_other": None,
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        self.assertRaises(ValidationError, form_validator.validate)

    def test_other_tissue_biopsy_org_with_tissue_biopsy_org_other(self):
        cleaned_data = {
            "subject_visit": self.subject_visit,
            "tissue_biopsy_organism": OTHER,
            "tissue_biopsy_organism_other": "some tissue organism",
        }
        form_validator = MicrobiologyFormValidator(cleaned_data=cleaned_data)
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got{e}")
